package com.gyee.generation.util.realtimesource;




import com.gyee.generation.model.auto.PointDataTaos;
import com.gyee.generation.model.auto.ProBasicEquipmentPoint;
import com.gyee.generation.model.auto.ProBasicPowerstationPoint;
import com.gyee.generation.util.realtimesource.timeseries.PointData;

import java.util.List;
import java.util.Map;

/**
 * @description:
 * @author: shilin
 * @date: 2024/5/31 9:06
 * @param: 
 * @return: 
 **/
public interface IRealDataUtil {


   /**
    * @description: 实时数据读取
    * @author: shilin
    * @date: 2024/6/5 10:15
    * @param: [point] 场站测点
    * @return: com.gyee.generation.util.realtimesource.timeseries.PointData
    **/
    public  PointData getRealData(ProBasicPowerstationPoint point) throws Exception;
    /**
     * @description: 实时数据读取
     * @author: shilin
     * @date: 2024/6/5 10:15
     * @param: [point] 设备测点
     * @return: com.gyee.generation.util.realtimesource.timeseries.PointData
     **/
    public PointData getRealData(ProBasicEquipmentPoint point) throws Exception ;
    /**
     * @description: 实时数据读取
     * @author: shilin
     * @date: 2024/6/5 10:15
     * @param: [point] 测点字符串
     * @return: com.gyee.generation.util.realtimesource.timeseries.PointData
     **/
    public PointData getRealData(String pointid) throws Exception ;
    /**
     * @description: 实时数据读取
     * @author: shilin
     * @date: 2024/6/5 10:15
     * @param: [point] 测点字符串数组
     * @return: com.gyee.generation.util.realtimesource.timeseries.PointData
     **/
    public List<PointData> getRealData(String... pointids) throws Exception ;

    /**
     * @description: 实时数据读取
     * @author: shilin
     * @date: 2024/6/5 10:15
     * @param: [point] 测点字符串集合
     * @return: com.gyee.generation.util.realtimesource.timeseries.PointData
     **/

    public List<PointData> getRealData(List<String> pointids) throws Exception;

    /**
     * @description: 实时数据读取,返回Map集合
     * @author: shilin
     * @date: 2024/6/5 10:15
     * @param: [point] 测点字符串数组
     * @return: com.gyee.generation.util.realtimesource.timeseries.PointData
     **/
    public Map<String, Double> getRealDataMap(String... pointids) throws Exception ;
    /**
     * @description: 快照查询
     * @author: shilin
     * @date: 2024/6/5 10:20
     * @param: [point 场站测点, beginDate 开始时间，单位是秒, endDate 结束时间，单位是秒, count 返回结果数量, pried 间隔步长，单位是秒]
     * @return: java.util.List<com.gyee.generation.util.realtimesource.timeseries.PointData>
     **/
    public List<PointData> getHistoryDatasSnap(ProBasicPowerstationPoint point, Long beginDate, Long endDate, Long count, Long pried) throws Exception ;
    /**
     * @description: 快照查询
     * @author: shilin
     * @date: 2024/6/5 10:20
     * @param: [point 设备测点, beginDate 开始时间，单位是秒, endDate 结束时间，单位是秒, count 返回结果数量, pried 间隔步长，单位是秒]
     * @return: java.util.List<com.gyee.generation.util.realtimesource.timeseries.PointData>
     **/
    public List<PointData> getHistoryDatasSnap(ProBasicEquipmentPoint point, Long beginDate, Long endDate, Long count, Long pried) throws Exception ;
    /**
     * @description: 快照查询
     * @author: shilin
     * @date: 2024/6/5 10:20
     * @param: [point 测点, beginDate 开始时间，单位是秒, endDate 结束时间，单位是秒, count 返回结果数量, pried 间隔步长，单位是秒]
     * @return: java.util.List<com.gyee.generation.util.realtimesource.timeseries.PointData>
     **/
    public List<PointData> getHistoryDatasSnap(String pointid, Long beginDate, Long endDate, Long count, Long pried) throws Exception;


    /**
     * @description: 快照历史数据
     * @author: shilin
     * @date: 2024/6/5 10:26
     * @param: [point 场站测点, beginDate 开始时间，单位是秒, endDate 结束时间，单位是秒]
     * @return: java.util.List<com.gyee.generation.util.realtimesource.timeseries.PointData>
     **/
    public List<PointData> getHistoryDatasRaw(ProBasicPowerstationPoint point, Long beginDate, Long endDate) throws Exception ;
    /**
     * @description: 快照历史数据
     * @author: shilin
     * @date: 2024/6/5 10:26
     * @param: [point 设备测点, beginDate 开始时间，单位是秒, endDate 结束时间，单位是秒]
     * @return: java.util.List<com.gyee.generation.util.realtimesource.timeseries.PointData>
     **/
    public List<PointData> getHistoryDatasRaw(ProBasicEquipmentPoint point, Long beginDate, Long endDate) throws Exception ;

    /**
     * @description: 快照历史数据
     * @author: shilin
     * @date: 2024/6/5 10:26
     * @param: [point 测点, beginDate 开始时间，单位是秒, endDate 结束时间，单位是秒]
     * @return: java.util.List<com.gyee.generation.util.realtimesource.timeseries.PointData>
     **/
    public List<PointData> getHistoryDatasRaw(String pointid, Long beginDate, Long endDate) throws Exception ;


    /**
     * @description: 统计数据
     * @author: shilin
     * @date: 2024/6/5 10:28
     * @param: [point 测点, beginDate 开始时间，单位是秒, endDate 结束时间，单位是秒, count 返回结果数量, pried 间隔步长，单位是秒,type 0 最大，1 最小，2 平均值 ]
     * @return: java.util.List<com.gyee.generation.util.realtimesource.timeseries.PointData>
     **/
    public List<PointData> getHistStat(ProBasicPowerstationPoint point, Long beginDate, Long endDate, Long count, Long pried, int type) throws Exception ;

    /**
     * @description: 统计数据
     * @author: shilin
     * @date: 2024/6/5 10:28
     * @param: [point 测点, beginDate 开始时间，单位是秒, endDate 结束时间，单位是秒, count 返回结果数量, pried 间隔步长，单位是秒,type 0 最大，1 最小，2 平均值 ]
     * @return: java.util.List<com.gyee.generation.util.realtimesource.timeseries.PointData>
     **/
    public List<PointData> getHistStat(ProBasicEquipmentPoint point, Long beginDate, Long endDate, Long count, Long pried, int type) throws Exception ;
    /**
     * @description: 统计数据
     * @author: shilin
     * @date: 2024/6/5 10:28
     * @param: [point 测点, beginDate 开始时间，单位是秒, endDate 结束时间，单位是秒, count 返回结果数量, pried 间隔步长，单位是秒,type 0 最大，1 最小，2 平均值 ]
     * @return: java.util.List<com.gyee.generation.util.realtimesource.timeseries.PointData>
     **/
    public List<PointData> getHistStat(String pointid, Long beginDate, Long endDate, Long count, Long pried, int type) throws Exception ;


    /**
     * @description: 添加和修改
     * @author: shilin
     * @date: 2024/6/5 10:32
     * @param: [taosList] 测点内容
     * @return: void
     **/
    public void sendSinglePoint(List<PointDataTaos>  taosList )  ;

    public void createSql(List<PointDataTaos>  taosList )   ;

    //多点切面数据


   /**
    * @description: 切面查询
    * @author: shilin
    * @date: 2024/6/5 10:33
    * @param: [pointids 测点编号集合, tTime 时间，单位是秒]
    * @return: java.util.List<com.gyee.generation.util.realtimesource.timeseries.PointData>
    **/
    public List<PointData> getHistMatrix(List<String> pointids, long tTime) throws Exception ;
    /**
     * @description: 切面查询
     * @author: shilin
     * @date: 2024/6/5 10:33
     * @param: [id 测点编号, tTime 时间，单位是秒]
     * @return: com.gyee.generation.util.realtimesource.timeseries.PointData
     **/
    public PointData getHistMatrix(String id, long tTime) throws Exception;
}
