package com.gyee.generation.util.realtimesource;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;


import com.gyee.generation.mapper.auto.TaosMapper;
import com.gyee.generation.model.auto.PointDataTaos;
import com.gyee.generation.model.auto.ProBasicEquipmentPoint;
import com.gyee.generation.model.auto.ProBasicPowerstationPoint;
import com.gyee.generation.util.realtimesource.timeseries.*;
import jakarta.annotation.Resource;
import org.springframework.beans.factory.annotation.Value;

import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.sql.SQLException;
import java.util.*;

/**
 * @description:
 * @author: shilin
 * @date: 2024/5/31 8:42
 * @param:
 * @return:
 **/
@Component
public class TaosUtil implements IRealDataUtil {

    private RestTemplate restTemplate = new RestTemplate();
    @Value("${db.url1}")
    private String baseURL;

    @Value("${db.url2}")
    private String baseURL2;

    @Value("${initialcode}")
    private String initialcode;

    private final Integer BAD_CODE=-99;
    @Resource
    private TaosMapper taosMapper;

    @Override
    public PointData getRealData(ProBasicPowerstationPoint point) throws Exception {


        if (ObjectUtil.isNotEmpty(point) && StrUtil.isNotEmpty(point.getNemCode()) && !initialcode.equals(point.getNemCode())) {

            List<PointDataTaos> pointls = taosMapper.getRealData(point.getNemCode());

            if (!pointls.isEmpty()) {
                List<PointData>  dataList = getPointData(point.getNemCode(), pointls);
                return dataList.get(0);
            }

        }
        PointData data = new PointData();
        data.setEdnaId(point.getNemCode());
        data.setPointTime(DateUtil.current());
        data.setPointValue("0");
        data.setPointValueInDouble(0.0);
        data.setMetric(BAD_CODE);
        return data;
    }

    private  List<PointData> getPointData(String point, List<PointDataTaos> pointls) {

        List<PointData> pointDataList=new ArrayList();

        if(!pointls.isEmpty())
        {
            for(PointDataTaos po:pointls)
            {
                if(ObjectUtil.isNotEmpty(po))
                {
                    PointData data = new PointData();
                    data.setEdnaId(point);
                    data.setPointTime(po.getTs().getTime());
                    data.setPointValueInDouble(po.getVal());
                    data.setMetric(po.getMetric());
                    pointDataList.add(data);
                }
            }


        }

        return pointDataList;
    }


    @Override
    public List<PointData> getHistoryDatasSnap(ProBasicPowerstationPoint point, Long beginDate, Long endDate, Long count, Long pried) throws Exception {

        if (ObjectUtil.isNotEmpty(point) && ObjectUtil.isNotEmpty(point.getNemCode()) && !initialcode.equals(point.getNemCode()) && ObjectUtil.isNotEmpty(pried)) {

            //通过时间区间和时间间隔获取点数
            Long finalInterval = pried;
            if (count != null)
            {
                finalInterval = (endDate - beginDate) / count;
            }else
            {
                count = (endDate - beginDate) / finalInterval;
            }

            // 选择数据库
            StringBuilder sb = new StringBuilder();

            sb.append(" SELECT LAST_ROW(ts) as ts,LAST_ROW(val) as  val ,LAST_ROW(metric) as metric FROM  ");
            sb.append(point.getNemCode());
            sb.append("  where ts>=").append(beginDate * 1000);
            sb.append(" and ts<=").append(endDate * 1000);
            sb.append(" INTERVAL(").append(finalInterval.intValue()).append("s)");
            sb.append(" FILL(PREV) ");

            List<PointDataTaos> pointls = taosMapper.getHistoryDatasSnap(String.valueOf(sb));
            if (!pointls.isEmpty()) {

                List<PointData>  dataList = getPointData(point.getNemCode(), pointls);
                if(!dataList.isEmpty() && dataList.size()==count)
                {
                    return dataList;
                } else   {
                    dataList =new ArrayList();
                    Calendar ca = Calendar.getInstance();
                    ca.setTimeInMillis(beginDate * 1000);
                    Date end = ca.getTime();
                    ca.add(Calendar.YEAR, -1);
                    Date begin = ca.getTime();
                    sb.setLength(0);
                    sb.append(" SELECT  LAST_ROW(ts) as ts,LAST_ROW(val) as  val ,LAST_ROW(metric) as metric FROM  ");
                    sb.append(point.getNemCode());
                    sb.append("  where ts>=").append(begin.getTime());
                    sb.append(" and ts<=").append(end.getTime());

                    pointls=  taosMapper.getHistMatrix(String.valueOf(sb));

                    ca.setTimeInMillis(beginDate * 1000);
                    double value = 0.0;
                    if (!pointls.isEmpty()) {

                        if(ObjectUtil.isNotEmpty(pointls.get(0)))
                        {
                            value=pointls.get(0).getVal();
                        }
                    }
                    if (pried != null && pried != 0) {
                        count = (endDate - beginDate) / pried;
                    }

                    for (int i = 0; i < count; i++) {
                        PointData po = new PointData();
                        po.setEdnaId(point.getNemCode());
                        po.setPointTime(ca.getTimeInMillis() / 1000);
                        po.setPointValueInDouble(value);
                        dataList.add(po);
                        ca.add(Calendar.SECOND, finalInterval.intValue());
                    }

                    return dataList;
                }

            }



        }

        List<PointData> pols = new ArrayList<>();
        PointData po = new PointData();
        po.setEdnaId(point.getNemCode());
        po.setPointValueInDouble(0.0);
        po.setPointTime(0l);
        po.setMetric(BAD_CODE);
        pols.add(po);
        return pols;
    }



    @Override
    public List<PointData> getHistoryDatasRaw(ProBasicPowerstationPoint point, Long beginDate, Long endDate) throws Exception {

        if (ObjectUtil.isNotEmpty(point) && ObjectUtil.isNotEmpty(point.getNemCode()) && !initialcode.equals(point.getNemCode()) ) {


            StringBuilder sb = new StringBuilder();

            sb.append(" SELECT ts,val,metric  FROM  ");
            sb.append(point.getNemCode());
            sb.append("  where ts>=").append(beginDate * 1000);
            sb.append(" and ts<=").append(endDate * 1000);

            List<PointDataTaos> pointls = taosMapper.getHistoryDatasRaw(String.valueOf(sb));
            if (!pointls.isEmpty()) {
                List<PointData>  dataList = getPointData(point.getNemCode(), pointls);
                return dataList;
            }
        }

        List<PointData> pols = new ArrayList<>();
        PointData po = new PointData();
        po.setEdnaId(point.getNemCode());
        po.setPointValueInDouble(0.0);
        po.setPointTime(0l);
        po.setMetric(BAD_CODE);
        pols.add(po);
        return pols;
    }

    @Override
    public PointData getRealData(ProBasicEquipmentPoint point) throws Exception {

        if (ObjectUtil.isNotEmpty(point) && StrUtil.isNotEmpty(point.getNemCode()) && !initialcode.equals(point.getNemCode())) {

            List<PointDataTaos> pointls = taosMapper.getRealData(point.getNemCode());

            if (!pointls.isEmpty()) {
                List<PointData>  dataList = getPointData(point.getNemCode(), pointls);
                return dataList.get(0);
            }

        }
        PointData data = new PointData();
        data.setEdnaId(point.getNemCode());
        data.setPointTime(DateUtil.current());
        data.setPointValue("0");
        data.setPointValueInDouble(0.0);
        data.setMetric(BAD_CODE);
        return data;

    }

    @Override
    public List<PointData> getHistoryDatasSnap(ProBasicEquipmentPoint point, Long beginDate, Long endDate, Long count, Long pried) throws Exception {

        if (ObjectUtil.isNotEmpty(point) && ObjectUtil.isNotEmpty(point.getNemCode()) && !initialcode.equals(point.getNemCode()) && ObjectUtil.isNotEmpty(pried)) {

            //通过时间区间和时间间隔获取点数
            Long finalInterval = pried;
            if (count != null)
            {
                finalInterval = (endDate - beginDate) / count;
            }else
            {
                count = (endDate - beginDate) / finalInterval;
            }

            // 选择数据库
            StringBuilder sb = new StringBuilder();

            sb.append(" SELECT LAST_ROW(ts) as ts,LAST_ROW(val) as  val ,LAST_ROW(metric) as metric  FROM  ");
            sb.append(point.getNemCode());
            sb.append("  where ts>=").append(beginDate * 1000);
            sb.append(" and ts<=").append(endDate * 1000);
            sb.append(" INTERVAL(").append(finalInterval.intValue()).append("s)");
            sb.append(" FILL(PREV) ");

            List<PointDataTaos> pointls = taosMapper.getHistoryDatasSnap(String.valueOf(sb));

            if (!pointls.isEmpty()) {

                List<PointData>  dataList = getPointData(point.getNemCode(), pointls);
                if(!dataList.isEmpty() && dataList.size()==count)
                {
                    return dataList;
                } else   {
                    dataList =new ArrayList();
                    Calendar ca = Calendar.getInstance();
                    ca.setTimeInMillis(beginDate * 1000);
                    Date end = ca.getTime();
                    ca.add(Calendar.YEAR, -1);
                    Date begin = ca.getTime();
                    sb.setLength(0);
                    sb.append(" SELECT  LAST_ROW(ts) as ts,LAST_ROW(val) as  val ,LAST_ROW(metric) as metric FROM  ");
                    sb.append(point.getNemCode());
                    sb.append("  where ts>=").append(begin.getTime());
                    sb.append(" and ts<=").append(end.getTime());

                    pointls=  taosMapper.getHistMatrix(String.valueOf(sb));

                    ca.setTimeInMillis(beginDate * 1000);
                    double value = 0.0;
                    if (!pointls.isEmpty()) {

                        if(ObjectUtil.isNotEmpty(pointls.get(0)))
                        {
                            value=pointls.get(0).getVal();
                        }
                    }
                    if (pried != null && pried != 0) {
                        count = (endDate - beginDate) / pried;
                    }

                    for (int i = 0; i < count; i++) {
                        PointData po = new PointData();
                        po.setEdnaId(point.getNemCode());
                        po.setPointTime(ca.getTimeInMillis() / 1000);
                        po.setPointValueInDouble(value);
                        dataList.add(po);
                        ca.add(Calendar.SECOND, finalInterval.intValue());
                    }

                    return dataList;
                }

            }



        }

        List<PointData> pols = new ArrayList<>();
        PointData po = new PointData();
        po.setEdnaId(point.getNemCode());
        po.setPointValueInDouble(0.0);
        po.setPointTime(0l);
        po.setMetric(BAD_CODE);
        pols.add(po);
        return pols;

    }


    @Override
    public List<PointData> getHistoryDatasRaw(ProBasicEquipmentPoint point, Long beginDate, Long endDate) throws Exception {
        if (ObjectUtil.isNotEmpty(point) && ObjectUtil.isNotEmpty(point.getNemCode()) && !initialcode.equals(point.getNemCode()) ) {


            StringBuilder sb = new StringBuilder();

            sb.append(" SELECT ts,val,metric FROM  ");
            sb.append(point.getNemCode());
            sb.append("  where ts>=").append(beginDate * 1000);
            sb.append(" and ts<=").append(endDate * 1000);

            List<PointDataTaos> pointls = taosMapper.getHistoryDatasRaw(String.valueOf(sb));
            if (!pointls.isEmpty()) {
                List<PointData>  dataList = getPointData(point.getNemCode(), pointls);
                return dataList;
            }
        }

        List<PointData> pols = new ArrayList<>();
        PointData po = new PointData();
        po.setEdnaId(point.getNemCode());
        po.setPointValueInDouble(0.0);
        po.setPointTime(0l);
        po.setMetric(BAD_CODE);
        pols.add(po);
        return pols;

    }

    @Override
    public PointData getRealData(String pointid) throws Exception {

        if (ObjectUtil.isNotEmpty(pointid) && StrUtil.isNotEmpty(pointid) && !initialcode.equals(pointid)) {

            List<PointDataTaos> pointls = taosMapper.getRealData(pointid);

            if (!pointls.isEmpty()) {
                List<PointData>  dataList = getPointData(pointid, pointls);
                return dataList.get(0);
            }

        }
        PointData data = new PointData();
        data.setEdnaId(pointid);
        data.setPointTime(DateUtil.current());
        data.setPointValue("0");
        data.setPointValueInDouble(0.0);
        data.setMetric(BAD_CODE);
        return data;

    }

    @Override
    public List<PointData> getRealData(String... pointids) throws Exception {


        if (ObjectUtil.isNotEmpty(pointids) && pointids.length>0) {
            List<PointData> pointls=new ArrayList();
            for(String pointId:pointids)
            {
                List<PointDataTaos> ls = taosMapper.getRealData(pointId);

                if (!ls.isEmpty()) {
                    List<PointData>  dataList = getPointData(pointId, ls);
                    pointls.add(dataList.get(0));

                }
            }

            return pointls;

        }

        List<PointData> pols = new ArrayList<>();
        PointData po = new PointData();
        po.setPointValueInDouble(0.0);
        po.setPointTime(0l);
        po.setMetric(BAD_CODE);
        pols.add(po);
        return pols;

    }


    @Override
    public List<PointData> getRealData(List<String> pointids) throws Exception {


        if (ObjectUtil.isNotEmpty(pointids) && !pointids.isEmpty()) {
            List<PointData> pointls=new ArrayList();
            for(String pointId:pointids)
            {
                List<PointDataTaos> ls = taosMapper.getRealData(pointId);

                if (!ls.isEmpty()) {
                    List<PointData>  dataList = getPointData(pointId, ls);
                    pointls.add(dataList.get(0));

                }
            }

            return pointls;

        }

        List<PointData> pols = new ArrayList<>();
        PointData po = new PointData();
        po.setPointValueInDouble(0.0);
        po.setPointTime(0l);
        po.setMetric(BAD_CODE);
        pols.add(po);
        return pols;


    }

    @Override
    public Map<String, Double> getRealDataMap(String... pointids) throws Exception {

        Map<String, Double> map=new HashMap<>();
        if (ObjectUtil.isNotEmpty(pointids) && pointids.length>0) {

            for(String pointId:pointids)
            {
                List<PointDataTaos> ls = taosMapper.getRealData(pointId);

                if (!ls.isEmpty()) {
                    List<PointData>  dataList = getPointData(pointId, ls);

                    map.put(dataList.get(0).getEdnaId(),dataList.get(0).getPointValueInDouble());


                }
            }

            return map;

        }

        map.put("BAD_CODE",BAD_CODE.doubleValue());

        return map;

    }


    @Override
    public List<PointData> getHistoryDatasSnap(String pointid, Long beginDate, Long endDate, Long count, Long pried) throws Exception {


        if (ObjectUtil.isNotEmpty(pointid) && ObjectUtil.isNotEmpty(pointid) && !initialcode.equals(pointid) ) {

            //通过时间区间和时间间隔获取点数
            Long finalInterval = pried;
            if (count != null)
            {
                finalInterval = (endDate - beginDate) / count;
            }else
            {
                count = (endDate - beginDate) / finalInterval;
            }

            // 选择数据库
            StringBuilder sb = new StringBuilder();

            sb.append("  SELECT LAST_ROW(ts) as ts,LAST_ROW(val) as  val ,LAST_ROW(metric) as metric  FROM  ");
            sb.append(pointid);
            sb.append("  where ts>=").append(beginDate * 1000);
            sb.append(" and ts<=").append(endDate * 1000);
            sb.append(" INTERVAL(").append(finalInterval.intValue()).append("s)");
            sb.append(" FILL(PREV) ");

            List<PointDataTaos> pointls = taosMapper.getHistoryDatasSnap(String.valueOf(sb));
            if (!pointls.isEmpty()) {

                List<PointData>  dataList = getPointData(pointid, pointls);
                if(!dataList.isEmpty() && dataList.size()==count)
                {
                    return dataList;
                } else   {
                   dataList =new ArrayList();
                    Calendar ca = Calendar.getInstance();
                    ca.setTimeInMillis(beginDate * 1000);
                    Date end = ca.getTime();
                    ca.add(Calendar.YEAR, -1);
                    Date begin = ca.getTime();
                    sb.setLength(0);
                    sb.append(" SELECT  LAST_ROW(ts) as ts,LAST_ROW(val) as  val ,LAST_ROW(metric) as metric FROM  ");
                    sb.append(pointid);
                    sb.append("  where ts>=").append(begin.getTime());
                    sb.append(" and ts<=").append(end.getTime());

                    pointls=  taosMapper.getHistMatrix(String.valueOf(sb));

                    ca.setTimeInMillis(beginDate * 1000);
                    double value = 0.0;
                    if (!pointls.isEmpty()) {

                        if(ObjectUtil.isNotEmpty(pointls.get(0)))
                        {
                            value=pointls.get(0).getVal();
                        }
                    }
                    if (pried != null && pried != 0) {
                        count = (endDate - beginDate) / pried;
                    }

                    for (int i = 0; i < count; i++) {
                        PointData po = new PointData();
                        po.setEdnaId(pointid);
                        po.setPointTime(ca.getTimeInMillis() / 1000);
                        po.setPointValueInDouble(value);
                        dataList.add(po);
                        ca.add(Calendar.SECOND, finalInterval.intValue());
                    }

                    return dataList;
                }

            }



        }

        List<PointData> pols = new ArrayList<>();
        PointData po = new PointData();
        po.setEdnaId(pointid);
        po.setPointValueInDouble(0.0);
        po.setPointTime(0l);
        po.setMetric(BAD_CODE);
        pols.add(po);
        return pols;

    }


    @Override
    public List<PointData> getHistoryDatasRaw(String pointid, Long beginDate, Long endDate) throws Exception {

        if (ObjectUtil.isNotEmpty(pointid) && ObjectUtil.isNotEmpty(pointid) && !initialcode.equals(pointid) ) {


            StringBuilder sb = new StringBuilder();

            sb.append(" SELECT ts,val,metric FROM  ");
            sb.append(pointid);
            sb.append("  where ts>=").append(beginDate * 1000);
            sb.append(" and ts<=").append(endDate * 1000);

            List<PointDataTaos> pointls = taosMapper.getHistoryDatasRaw(String.valueOf(sb));
            if (!pointls.isEmpty()) {

                List<PointData>  dataList = getPointData(pointid, pointls);
                return dataList;
            }
        }

        List<PointData> pols = new ArrayList<>();
        PointData po = new PointData();
        po.setEdnaId(pointid);
        po.setPointValueInDouble(0.0);
        po.setPointTime(0l);
        po.setMetric(BAD_CODE);
        pols.add(po);
        return pols;


    }

    @Override
    public List<PointData> getHistStat(ProBasicEquipmentPoint point, Long beginDate, Long endDate, Long count, Long pried, int type) throws Exception {

        if (ObjectUtil.isNotEmpty(point.getNemCode()) && ObjectUtil.isNotEmpty(point.getNemCode()) && !initialcode.equals(point.getNemCode()) ) {

//通过时间区间和时间间隔获取点数
            Long finalInterval = pried;
            if (count != null)
            {
                finalInterval = (endDate - beginDate) / count;
            }else
            {
                count = (endDate - beginDate) / finalInterval;
            }
            StringBuilder sb = new StringBuilder();

            sb.append(" SELECT ts,");
            switch (type) {
                case 0:
                    sb.append("MAX");
                    break;
                case 1:
                    sb.append("MIN");
                    break;
                case 2:
                    sb.append("AVG");
                    break;
                default:
                    break;
            }

            sb.append("(val) as  val ,metric FROM  ");
            sb.append(point.getNemCode());
            sb.append("  where ts>=").append(beginDate * 1000);
            sb.append(" and ts<=").append(endDate * 1000);
            sb.append(" INTERVAL(").append(finalInterval).append("s)");
            sb.append(" FILL(PREV) ");

            List<PointDataTaos> pointls = taosMapper.getHistoryDatasSnap(String.valueOf(sb));
            if (!pointls.isEmpty()) {
                List<PointData>  dataList = getPointData(point.getNemCode(), pointls);
                if(!dataList.isEmpty() && dataList.size()==count)
                {
                    return dataList;
                } else   {
                    dataList =new ArrayList();
                    Calendar ca = Calendar.getInstance();
                    ca.setTimeInMillis(beginDate * 1000);
                    Date end = ca.getTime();
                    ca.add(Calendar.YEAR, -1);
                    Date begin = ca.getTime();
                    sb.setLength(0);
                    sb.append(" SELECT  LAST_ROW(ts) as ts,LAST_ROW(val) as  val ,LAST_ROW(metric) as metric FROM  ");
                    sb.append(point.getNemCode());
                    sb.append("  where ts>=").append(begin.getTime());
                    sb.append(" and ts<=").append(end.getTime());

                    pointls=  taosMapper.getHistMatrix(String.valueOf(sb));

                    ca.setTimeInMillis(beginDate * 1000);
                    double value = 0.0;
                    if (!pointls.isEmpty()) {

                        if(ObjectUtil.isNotEmpty(pointls.get(0)))
                        {
                            value=pointls.get(0).getVal();
                        }
                    }
                    if (pried != null && pried != 0) {
                        count = (endDate - beginDate) / pried;
                    }

                    for (int i = 0; i < count; i++) {
                        PointData po = new PointData();
                        po.setEdnaId(point.getNemCode());
                        po.setPointTime(ca.getTimeInMillis() / 1000);
                        po.setPointValueInDouble(value);
                        dataList.add(po);
                        ca.add(Calendar.SECOND, finalInterval.intValue());
                    }

                    return dataList;
                }

            }
        }

        List<PointData> pols = new ArrayList<>();
        PointData po = new PointData();
        po.setEdnaId(point.getNemCode());
        po.setPointValueInDouble(0.0);
        po.setPointTime(0l);
        po.setMetric(BAD_CODE);
        pols.add(po);
        return pols;

    }

    @Override
    public List<PointData> getHistStat(ProBasicPowerstationPoint point, Long beginDate, Long endDate, Long count, Long pried, int type) throws Exception {

        if (ObjectUtil.isNotEmpty(point.getNemCode()) && ObjectUtil.isNotEmpty(point.getNemCode()) && !initialcode.equals(point.getNemCode()) ) {

//通过时间区间和时间间隔获取点数
            Long finalInterval = pried;
            if (count != null)
            {
                finalInterval = (endDate - beginDate) / count;
            }else
            {
                count = (endDate - beginDate) / finalInterval;
            }
            StringBuilder sb = new StringBuilder();

            sb.append(" SELECT ts,");
            switch (type) {
                case 0:
                    sb.append("MAX");
                    break;
                case 1:
                    sb.append("MIN");
                    break;
                case 2:
                    sb.append("AVG");
                    break;
                default:
                    break;
            }

            sb.append("(val) as  val ,metric FROM  ");
            sb.append(point.getNemCode());
            sb.append("  where ts>=").append(beginDate * 1000);
            sb.append(" and ts<=").append(endDate * 1000);
            sb.append(" INTERVAL(").append(finalInterval).append("s)");
            sb.append(" FILL(PREV) ");

            List<PointDataTaos> pointls = taosMapper.getHistoryDatasSnap(String.valueOf(sb));
            if (!pointls.isEmpty()) {
                List<PointData>  dataList = getPointData(point.getNemCode(), pointls);
                if(!dataList.isEmpty() && dataList.size()==count)
                {
                    return dataList;
                } else   {
                    dataList =new ArrayList();
                    Calendar ca = Calendar.getInstance();
                    ca.setTimeInMillis(beginDate * 1000);
                    Date end = ca.getTime();
                    ca.add(Calendar.YEAR, -1);
                    Date begin = ca.getTime();
                    sb.setLength(0);
                    sb.append(" SELECT  LAST_ROW(ts) as ts,LAST_ROW(val) as  val ,LAST_ROW(metric) as metric FROM  ");
                    sb.append(point.getNemCode());
                    sb.append("  where ts>=").append(begin.getTime());
                    sb.append(" and ts<=").append(end.getTime());

                    pointls=  taosMapper.getHistMatrix(String.valueOf(sb));

                    ca.setTimeInMillis(beginDate * 1000);
                    double value = 0.0;
                    if (!pointls.isEmpty()) {

                        if(ObjectUtil.isNotEmpty(pointls.get(0)))
                        {
                            value=pointls.get(0).getVal();
                        }
                    }
                    if (pried != null && pried != 0) {
                        count = (endDate - beginDate) / pried;
                    }

                    for (int i = 0; i < count; i++) {
                        PointData po = new PointData();
                        po.setEdnaId(point.getNemCode());
                        po.setPointTime(ca.getTimeInMillis() / 1000);
                        po.setPointValueInDouble(value);
                        dataList.add(po);
                        ca.add(Calendar.SECOND, finalInterval.intValue());
                    }

                    return dataList;
                }
            }
        }

        List<PointData> pols = new ArrayList<>();
        PointData po = new PointData();
        po.setEdnaId(point.getNemCode());
        po.setPointValueInDouble(0.0);
        po.setPointTime(0l);
        po.setMetric(BAD_CODE);
        pols.add(po);
        return pols;
    }

    @Override
    public List<PointData> getHistStat(String pointid, Long beginDate, Long endDate, Long count, Long pried, int type) throws Exception {


        if (ObjectUtil.isNotEmpty(pointid) && ObjectUtil.isNotEmpty(pointid) && !initialcode.equals(pointid) ) {

//通过时间区间和时间间隔获取点数
            Long finalInterval = pried;
            if (count != null)
            {
                finalInterval = (endDate - beginDate) / count;
            }else
            {
                count = (endDate - beginDate) / finalInterval;
            }
            StringBuilder sb = new StringBuilder();

            sb.append(" SELECT ts,");
            switch (type) {
                case 0:
                    sb.append("MAX");
                    break;
                case 1:
                    sb.append("MIN");
                    break;
                case 2:
                    sb.append("AVG");
                    break;
                default:
                    break;
            }

            sb.append("(val) as  val ,metric FROM  ");
            sb.append(pointid);
            sb.append("  where ts>=").append(beginDate * 1000);
            sb.append(" and ts<=").append(endDate * 1000);
            sb.append(" INTERVAL(").append(finalInterval).append("s)");
            sb.append(" FILL(PREV) ");

            List<PointDataTaos> pointls = taosMapper.getHistoryDatasSnap(String.valueOf(sb));
            if (!pointls.isEmpty()) {
                List<PointData>  dataList = getPointData(pointid, pointls);
                if(!dataList.isEmpty() && dataList.size()==count)
                {
                    return dataList;
                } else   {
                    dataList =new ArrayList();
                    Calendar ca = Calendar.getInstance();
                    ca.setTimeInMillis(beginDate * 1000);
                    Date end = ca.getTime();
                    ca.add(Calendar.YEAR, -1);
                    Date begin = ca.getTime();
                    sb.setLength(0);
                    sb.append(" SELECT  LAST_ROW(ts) as ts,LAST_ROW(val) as  val ,LAST_ROW(metric) as metric FROM  ");
                    sb.append(pointid);
                    sb.append("  where ts>=").append(begin.getTime());
                    sb.append(" and ts<=").append(end.getTime());

                    pointls=  taosMapper.getHistMatrix(String.valueOf(sb));

                    ca.setTimeInMillis(beginDate * 1000);
                    double value = 0.0;
                    if (!pointls.isEmpty()) {

                        if(ObjectUtil.isNotEmpty(pointls.get(0)))
                        {
                            value=pointls.get(0).getVal();
                        }
                    }
                    if (pried != null && pried != 0) {
                        count = (endDate - beginDate) / pried;
                    }

                    for (int i = 0; i < count; i++) {
                        PointData po = new PointData();
                        po.setEdnaId(pointid);
                        po.setPointTime(ca.getTimeInMillis() / 1000);
                        po.setPointValueInDouble(value);
                        dataList.add(po);
                        ca.add(Calendar.SECOND, finalInterval.intValue());
                    }

                    return dataList;
                }
            }
        }
        List<PointData> pols = new ArrayList<>();
        PointData po = new PointData();
        po.setEdnaId(pointid);
        po.setPointValueInDouble(0.0);
        po.setPointTime(0l);
        po.setMetric(BAD_CODE);
        pols.add(po);
        return pols;

    }



    public void sendSinglePoint(List<PointDataTaos>  taosList )  {

        if(ObjectUtil.isNotEmpty(taosList) && !taosList.isEmpty())
        {

            createSql(taosList);
        }


    }

    public void createSql(List<PointDataTaos>  taosList )   {

        StringBuilder sb = new StringBuilder("INSERT INTO ");

        int i=0;
        for (PointDataTaos tag:taosList) {

            String tableName = tag.getId();
            StringBuilder sub_sb = new StringBuilder(tableName);

            sub_sb.append( " values ");
            sub_sb.append(" (");
            sub_sb.append(tag.getTs().getTime()).append(",").append(tag.getVal()).append(",").append(tag.getMetric());
            sub_sb.append(" ) ");
            i++;
            if (i==100) {
                taosMapper.insertList(String.valueOf(sb));
                sb = new StringBuilder("INSERT INTO ");
                i=0;
            }
            sb.append(sub_sb);

        }

        if(i!=0)
        {
            taosMapper.insertList(String.valueOf(sb));
        }

    }



    //多点切面数据


    @Override
    public List<PointData> getHistMatrix(List<String> pointids, long tTime) throws Exception {
        if (ObjectUtil.isNotEmpty(pointids) && !pointids.isEmpty() && ObjectUtil.isNotEmpty(tTime)) {
            List<PointData> pointls=new ArrayList();
            for(String pointId:pointids)
            {

                StringBuilder sb = new StringBuilder();
                sb.append("  SELECT LAST_ROW(ts) as ts,LAST_ROW(val) as  val ,LAST_ROW(metric) as metric  FROM ").append(pointId);
                sb.append("  where  ts>='").append(tTime*1000);
                sb.append("' and ts<='").append(tTime*1000).append("' ");
                sb.append(" FILL(PREV) ");

                List<PointDataTaos> ls = taosMapper.getHistMatrix(String.valueOf(sb));

                if (!ls.isEmpty()) {
                    List<PointData>  dataList = getPointData(pointId, ls);
                    pointls.add(dataList.get(0));

                }
            }

            return pointls;

        }

        List<PointData> pols = new ArrayList<>();
        PointData po = new PointData();
        po.setPointValueInDouble(0.0);
        po.setPointTime(0l);
        po.setMetric(BAD_CODE);
        pols.add(po);
        return pols;
    }

    @Override
    public PointData getHistMatrix(String id, long tTime) throws Exception {

        if (ObjectUtil.isNotEmpty(id) && !id.isEmpty()) {

            PointData po=new PointData();
            StringBuilder sb = new StringBuilder();
            sb.append("  SELECT LAST_ROW(ts) as ts,LAST_ROW(val) as  val ,LAST_ROW(metric) as metric  FROM ").append(id);
            sb.append("  where  ts>='").append(tTime*1000);
            sb.append("' and ts<='").append(tTime*1000).append("' ");
            sb.append(" FILL(PREV) ");

            List<PointDataTaos> ls = taosMapper.getHistMatrix(String.valueOf(sb));

            if (!ls.isEmpty()) {
                List<PointData>  dataList = getPointData(id, ls);
                po=dataList.get(0);

            }

            return po;

        }

        List<PointData> pols = new ArrayList<>();
        PointData po = new PointData();
        po.setPointValueInDouble(0.0);
        po.setPointTime(0l);
        po.setMetric(BAD_CODE);
        pols.add(po);
        return po;

    }


}
